/*# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Sant�)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
# Previous Authors : Laurent Guigues, Jean-Pierre Roux
# CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------ */ 

#ifndef ITKVTKCOLORTRANSFERFUNCTION_TXX
#define ITKVTKCOLORTRANSFERFUNCTION_TXX

namespace itk{


template< class TypeImage>
VTKColorTransferFunction<TypeImage>::VTKColorTransferFunction()
{

}

template< class TypeImage>
VTKColorTransferFunction<TypeImage>::~VTKColorTransferFunction()
{
    m_Output->Delete();
}

template< class TypeImage>
void VTKColorTransferFunction<TypeImage>::Update(){
    GenerateOutputInformation();
    GenerateData();
}

template< class TypeImage>
void VTKColorTransferFunction<TypeImage>::GenerateOutputInformation(){
     m_Output = vtkColorTransferFunction::New();
}



template< class TypeImage>
void VTKColorTransferFunction<TypeImage>::GenerateData(){



    InputImagePointerType imagergb = this->GetInput();

    if(!imagergb){
        itkExceptionMacro(<<"Set the image first!");
    }


    typedef itk::ImageRegionConstIterator< InputImageType > rgbiteratorType;
    rgbiteratorType rgbit(imagergb, imagergb->GetLargestPossibleRegion());


    rgbit.GoToBegin();


    m_Output->RemoveAllPoints ();


    while(!rgbit.IsAtEnd()){


        typename InputImageType::PixelType::ComponentType lum = 0.299*rgbit.Get()[0] + 0.587*rgbit.Get()[1] + 0.114*rgbit.Get()[2];

        m_Output->AddRGBPoint(lum, rgbit.Get()[0]/255.0, rgbit.Get()[1]/255.0, rgbit.Get()[2]/255.0);

        ++rgbit;        

    }




}

}
#endif
